#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <assert.h>
#include <GL/glut.h>
#include "object.h"
#include "matrix.h"

bool init();
void display();
void idle();
void reshape(int x, int y);
void keyb(unsigned char key, int x, int y);
void mouse(int bn, int state, int x, int y);
void motion(int x, int y);

float cam_theta, cam_phi = 25;
float cam_dist = 10;

Object *red_box, *green_box;

int main(int argc, char **argv)
{
	glutInit(&argc, argv);
	glutInitWindowSize(800, 600);
	glutInitDisplayMode(GLUT_RGB | GLUT_DEPTH | GLUT_DOUBLE);
	glutCreateWindow("foo");

	glutDisplayFunc(display);
	glutIdleFunc(idle);
	glutReshapeFunc(reshape);
	glutKeyboardFunc(keyb);
	glutMouseFunc(mouse);
	glutMotionFunc(motion);

	if(!init()) {
		return 1;
	}

	glutMainLoop();
	return 0;
}

bool init()
{
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_CULL_FACE);
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glEnable(GL_NORMALIZE);

	red_box = new Object;
	red_box->set_color(1, 0, 0);

	green_box = new Object;
	green_box->set_color(0, 1, 0);
	green_box->set_position(0, 2, 0);

	return true;
}

void display()
{
	float sec = (float)glutGet(GLUT_ELAPSED_TIME) / 1000.0;
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	Matrix4x4 modelview;
	modelview.translate(0, 0, -cam_dist);
	modelview.rotate(cam_phi, 1, 0, 0);
	modelview.rotate(cam_theta, 0, 1, 0);

	glMatrixMode(GL_MODELVIEW);
	glLoadMatrixf(modelview.m[0]);

	//red_box->set_position(sin(sec * 5.0), 0, 0);
	red_box->set_rotation(sec * 100.0, 0, 0, 1);
	green_box->set_rotation(sec * 100.0, 0, 0, 1);
	red_box->update();
	green_box->update();

	red_box->draw();
	green_box->draw();

	glutSwapBuffers();
	assert(glGetError() == GL_NO_ERROR);
}

void idle()
{
	glutPostRedisplay();
}

void reshape(int x, int y)
{
	glViewport(0, 0, x, y);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	Matrix4x4 proj;
	proj.perspective(45.0, (float)x / (float)y, 0.5, 500.0);
	glLoadMatrixf(proj.m[0]);
}

void keyb(unsigned char key, int x, int y)
{
	switch(key) {
	case 27:
		exit(0);
	}
}

static int prev_x, prev_y;
static bool bnstate[32];

void mouse(int bn, int state, int x, int y)
{
	int idx = bn - GLUT_LEFT_BUTTON;
	if(idx < (int)(sizeof bnstate / sizeof *bnstate)) {
		bnstate[idx] = state == GLUT_DOWN;
	}
	prev_x = x;
	prev_y = y;
}

void motion(int x, int y)
{
	int dx = x - prev_x;
	int dy = y - prev_y;
	prev_x = x;
	prev_y = y;

	if(bnstate[0]) {
		cam_theta += dx * 0.5;
		cam_phi += dy * 0.5;

		if(cam_phi < -90) {
			cam_phi = -90;
		}
		if(cam_phi > 90) {
			cam_phi = 90;
		}
		glutPostRedisplay();
	}
	if(bnstate[2]) {
		cam_dist += dy * 0.1;
		if(cam_dist < 0) {
			cam_dist = 0;
		}
		glutPostRedisplay();
	}
}
