#include <GL/glut.h>
#include "object.h"

static void draw_box();

Object::Object()
{
	rot_axis = Vector3(0, 0, 1);
	rot_angle = 0;
	scale = Vector3(1, 1, 1);

	color[0] = color[1] = color[2] = color[3] = 1.0;
}

Object::~Object()
{
}

void Object::set_color(float r, float g, float b)
{
	color[0] = r;
	color[1] = g;
	color[2] = b;
}

void Object::set_position(float x, float y, float z)
{
	pos = Vector3(x, y, z);
}

void Object::set_rotation(float angle, float x, float y, float z)
{
	rot_angle = angle;
	rot_axis = Vector3(x, y, z);
}

void Object::set_scaling(float sx, float sy, float sz)
{
	scale = Vector3(sx, sy, sz);
}

void Object::update()
{
	matrix.set_identity();
	matrix.translate(pos.x, pos.y, pos.z);
	matrix.rotate(rot_angle, rot_axis.x, rot_axis.y, rot_axis.z);
	matrix.scale(scale.x, scale.y, scale.z);
}

void Object::draw() const
{
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glMultMatrixf(matrix.m[0]);

	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, color);
	draw_box();

	glPopMatrix();
}

static void draw_box()
{
	glPushMatrix();
	glScalef(0.5, 2, 0.5);
	glTranslatef(0, 0.5, 0);
	glRotatef(45, 0, 1, 0);

	glutSolidCube(1.0);
	glPopMatrix();
}
